public class LabeledImage {
	/* Constant defining the length of one side of the image. */
	public static final int IMAGE_DIMENSION = 28;
	
	/* Constant defining the total number of pixels in each
	 * image.
	 */
	public static final int IMAGE_SIZE = IMAGE_DIMENSION * IMAGE_DIMENSION;
	
	/* The intensities of each pixel in the image. */
	private double[] pixels;
	
	/* The label associated with this image. */
	private int label;
	
	/**
	 * Given a string representation of the pixels in an image
	 * and the label for that image, constructs a new LabeledImage
	 * object.
	 * 
	 * @param pixel A string representation of the pixels in the image.
	 * @param label The label associated with that image.
	 */
	public LabeledImage(String pixelData, int imageLabel) {
		/* Split the pixel data apart. */
		String[] eachPixel = pixelData.split(" ");
		
		/* Convert each string representation of a double into an
		 * actual double value.  We also append a +1 to account
		 * for the constant term.
		 */
		pixels = new double[IMAGE_SIZE + 1];
		for (int i = 0; i < eachPixel.length; i++) {
			pixels[i] = Double.parseDouble(eachPixel[i]);
		}
		pixels[IMAGE_SIZE] = +1.0;
		
		label = imageLabel;
	}
	
	/**
	 * Returns the label associated with this image.
	 * 
	 * @return The label associated with this image.
	 */
	public int getLabel() {
		return label;
	}
	
	/**
	 * Returns the array of the pixels in this image.
	 * 
	 * @return THe array of the pixels in this image.
	 */
	public double[] getPixels() {
		return pixels;
	}
}
