import java.util.*;
import javax.swing.*;
import acm.graphics.*;
import java.awt.*;

public class PerceptronWindow extends JFrame {
	/* The canvas where we will render the perceptron. */
	private GCanvas canvas;
	
	/* How large each pixel should be. */
	private static final int PIXEL_SIZE = 10;
	
	/**
	 * Creates a new PerceptronWindow to visualize a given
	 * perceptron.
	 * 
	 * @param perceptron The perceptron to visualize.
	 * @param digit Which digit it represents.
	 */
	public PerceptronWindow(double[] perceptron, int digit) {
		setTitle("Perceptron for " + digit);
		canvas = new GCanvas();
		add(canvas);
		setSize(LabeledImage.IMAGE_DIMENSION * PIXEL_SIZE,
				LabeledImage.IMAGE_DIMENSION * PIXEL_SIZE);
		
		setupCanvas(perceptron);
	}
	
	/**
	 * Sets up the canvas to display the given perceptron.
	 * 
	 * @param perceptron The perceptron to visualize.
	 */
	private void setupCanvas(double[] perceptron) {
		double max = maxOf(perceptron);
		double min = minOf(perceptron);
		
		for (int x = 0; x < LabeledImage.IMAGE_DIMENSION; x++) {
			for (int y = 0; y < LabeledImage.IMAGE_DIMENSION; y++) {
				plotPixel(x, y, perceptron[y * LabeledImage.IMAGE_DIMENSION + x], min, max);
			}
		}
	}
	
	/**
	 * Returns the maximum value in an array of doubles.
	 * 
	 * @param array The array in question.
	 * @return The maximum value.
	 */
	private double maxOf(double[] array) {
		double result = Double.NEGATIVE_INFINITY;
		for (double value: array) {
			result = Math.max(result, value);
		}
		return result;
	}
	
	/**
	 * Returns the minimum value in an array of doubles.
	 * 
	 * @param array The array in question.
	 * @return The minimum value.
	 */
	private double minOf(double[] array) {
		double result = Double.POSITIVE_INFINITY;
		for (double value: array) {
			result = Math.min(result, value);
		}
		return result;
	}
	
	/**
	 * Plots a single pixel on the canvas.
	 * 
	 * @param x The x coordinate of the pixel.
	 * @param y The y coordinate of the pixel.
	 * @param intensity The intensity of the pixel.
	 * @param minIntensity The minimum intensity.
	 * @param maxIntensity The maximum intensity.
	 */
	private void plotPixel(int x, int y, double intensity, double minIntensity, double maxIntensity) {
		/* Determine the relative intensity. */
		int rel = (int)(255 * ((intensity - minIntensity) / (maxIntensity - minIntensity)));
		
		/* Convert that to a Color object. */
		Color c = new Color(rel, rel, rel);
		
		/* Create a rectangle of that color. */
		GRect rect = new GRect(x * PIXEL_SIZE, y * PIXEL_SIZE, PIXEL_SIZE, PIXEL_SIZE);
		rect.setFilled(true);
		rect.setColor(c);
		canvas.add(rect);
	}
}
